;;-----------------------------LICENSE NOTICE------------------------------------
;;   AUTHORS OF THE GAME:
;;
;; - Andres David Rojas Carrillo  <http://andresrojas.tk>
;; - Mario Villora Pico
;; - Roberto Ruiz Ucles
;;
;;    This file is part of THEHERO

;;   THEHERO is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;   (at your option) any later version.

;;    THEHERO is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.

;;    You should have received a copy of the GNU General Public License
;;    along with Foobar.  If not, see <https://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------



;; Include all CPCtelera constant definitions, macros and variables
.include "cpctelera.h.s"
;;
;; Start of _DATA area 
;;  SDCC requires at least _DATA and _CODE areas to be declared, but you may use
;;  any one of them for any purpose. Usually, compiler puts _DATA area contents
;;  right after _CODE area contents.
;;
.area _DATA


;;
;; Start of _CODE area
;; 
.area _CODE

;; 
;; Declare all function entry points as global symbols for the compiler.
;; (The linker will know what to do with them)
;; WARNING: Every global symbol declared will be linked, so DO NOT declare 
;; symbols for functions you do not use.
;;
.globl cpct_disableFirmware_asm
.globl cpct_getScreenPtr_asm


;;
;; MAIN function. This is the entry point of the application.
;;    _main:: global symbol is required for correctly compiling and linking
;;

_final_muertes: .asciz "HAS MUERTO" 
_numero_muertes:: .db 48 , 48 , 48 , 48 , 00

_musica:: .db 0

_main::
   ;;CAMBIAMOS LA PILA DE POSICION 
   ld    sp, #0xA400

   ;;LLAMAMOS A CAMBIAR INTERRUPCIONES
   call  set_init_handler

   ;;INICIALIZAR MUSICA 
   call  music_init

   ;;INICIALIZAR RENDER 
   call rendersys_init
   
   ;; Init systems
   call  man_menu_init
      
   call  man_menu_update
   
   ret


loop_juego::
  
loop:
   call    ia_update 
   call    physics_update
   call    physics_update_bala
   call    colision_update

esperar:   
   ld   a , (cont_i)
   cp #6
   jr nz, esperar
   call    rendersys_update 
   call    render_death_update
  

   call     entity_update 

   ;SKIP LEVEL 
   ;ld    hl, #Key_N
   ;call  cpct_isKeyPressed_asm
   ;jr z, continuar_next

   ;call  next_level

;continuar_next:
   ;MUTE 
   ld    hl, #Key_M 
   call  cpct_isKeyPressed_asm
   jr z, continuar

   ld   a,  (_musica)  
   cp   #0 
   jr   nz , poner_play
   
   call cpct_akp_stop_asm
   ld    a, #1 
   ld   (_musica), a ;; 1 = mute

   jp continuar
poner_play:
   ld    a, #0
   ld   (_musica), a ;; 1 = mute
   

continuar:
   ;;PAUSA
   ld   hl, #Key_Esc
   call cpct_isKeyPressed_asm

   jr z, loop

   ld hl, #_screenpause_z_end
   ld de, #0xFFFF
   call cpct_zx7b_decrunch_s_asm


_final_loop:
   ld   hl, #Key_Esc
   call cpct_isKeyPressed_asm

   jr z, continuamos

   jp volvemos

continuamos:

   ;call cpct_scanKeyboard_f_asm
   ld   hl, #Key_R
   call cpct_isKeyPressed_asm
   jr z, _final_loop

   call finalizamos
   call _main

volvemos:

   ;;CAMBIAR RENDER TILE MAP0
   ld de, (_tile_map_colision)
   call render_tilemap_0
  


   ;;CAMBIAR RENDER TILE MAP1
   ld de, (_tile_map_colision)
   call render_tilemap_1


   ;;CAMBIAR RENDER TILE MAP2
   ld de, (_tile_map_colision)
   call render_tilemap_2


   ;;CAMBIAR RENDER TILE MAP3
   ld de, (_tile_map_colision)
   call render_tilemap_3

   call rendersys_pause_update 

   jp loop

   ret


game_over::

   call eliminar_todo

   ld hl, #_screenover_z_end
   ld de, #0xFFFF
   call cpct_zx7b_decrunch_s_asm

game_over2:
   ld   hl, #Key_R
   call cpct_isKeyPressed_asm

   jr z, game_over2

   call _main

   ret  



eliminar_todo:: 

    call  entity_man_getEntityArray_IX  ;; array de entidades  -> IX 
    
_desloop2:

    ld a, 0(ix) ;; cargo tipo de entidad  0000 0011
    or a ;; resto los tipos               0000 0011 
    jr z , actualizar_entidades;;                              0000 0011

    ld a, #_e_type_dead
    ld 0(ix), a 

    ld      bc,#entity_size ;; carga la cantidad de atributos que tiene entity  en bc 
    add     ix, bc  
    jr       _desloop2

actualizar_entidades: 
   call entity_update

   ret 


ganar::

   call eliminar_todo

   ld hl, #_screenganar_z_end
   ld de, #0xFFFF
   call cpct_zx7b_decrunch_s_asm

   ld hl , #0x000F
   call cpct_setDrawCharM0_asm

   ld iy , #_final_muertes
   ld hl , #0xC0B3
   call my_drawStringMO

   call  cargar_numero_muertes 

   ld iy , #_numero_muertes                        
   ld hl , #0xC15F
   call my_drawStringMO

ganar_loop:

   ld   hl, #Key_Return
   call cpct_isKeyPressed_asm

   jr z,    ganar_loop
   
   call     finalizamos
   call     man_menu_creditos
   call     wait_esc 


   ret

finalizamos::


   ld a, #0
   ld (_IndiceNiveles),a 
   ld (_IndiceMusica), a 
   ld hl , #_contador_vidas

   ld (hl), a 
   inc hl 
   ld (hl), a
   inc hl 
   ld (hl), a 
   inc hl 
   ld (hl), a 

   ret  

cargar_numero_muertes::
   ld  hl , #_numero_muertes

   ld a, #48
   ld (hl), a 
   inc hl 
   ld (hl), a
   inc hl 
   ld (hl), a 
   inc hl 
   ld (hl), a 
   
   ld  hl , #_numero_muertes
   ld  bc , #_contador_vidas

   ld a , (hl)
   ld e , a 
   ld a , (bc) 
   ld d , a 
   ld a , e 
   add d 

   ld  (hl) , a ;; millar  

   inc hl 
   inc bc 

   ld a , (hl)
   ld e , a 
   ld a , (bc) 
   ld d , a 
   ld a , e 
   add d 

   ld  (hl) , a ;; centenas 

   inc hl 
   inc bc 
   
   ld a , (hl)
   ld e , a 
   ld a , (bc) 
   ld d , a 
   ld a , e 
   add d 

   ld  (hl) , a ;; decenas

   inc hl 
   inc bc 
   
   ld a , (hl)
   ld e , a 
   ld a , (bc) 
   ld d , a 
   ld a , e 
   add d 

   ld  (hl) , a ;; unidades 

   ret